@@ -15,11 +15,15 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 #include "defs.h"
 #include "frame.h"
 #include "value.h"
+#include "varobj.h"
+#include "block.h"
+#include "exceptions.h"
 #include "gdbtk-wrapper.h"
 
 /*
@@ -78,6 +82,10 @@ gdb_result GDB_find_relative_frame (struct frame_info *fi,
 
 gdb_result GDB_get_current_frame (struct frame_info **result);
 
+gdb_result GDB_varobj_update (struct varobj **varp,
+			      struct varobj ***changelist, int explicit,
+			      int *result);
+
 /*
  * Private functions for this file
  */
@@ -123,6 +131,8 @@ static int wrap_get_next_frame (char *opaque_arg);
 static int wrap_find_relative_frame (char *opaque_arg);
 
 static int wrap_get_current_frame (char *opaque_arg);
+
+static int wrap_varobj_update (char *opaque_arg);
 
 static gdb_result
 call_wrapped_function (catch_errors_ftype *fn, struct gdb_wrapper_arguments *arg)
@@ -142,10 +152,10 @@ GDB_type_print (value_ptr val, char *varstring,
 {
   struct gdb_wrapper_arguments args;
 
-  args.args[0] = (char *) val;
-  args.args[1] = varstring;
-  args.args[2] = (char *) stream;
-  args.args[3] = (char *) show;
+  args.args[0].ptr = val;
+  args.args[1].ptr = varstring;
+  args.args[2].ptr = stream;
+  args.args[3].integer = show;
   return call_wrapped_function ((catch_errors_ftype *) wrap_type_print, &args);
 }
 
@@ -153,11 +163,11 @@ static int
 wrap_type_print (char *a)
 {
   struct gdb_wrapper_arguments **args = (struct gdb_wrapper_arguments **) a;
-  value_ptr val = (value_ptr) (*args)->args[0];
-  char *varstring = (*args)->args[1];
-  struct ui_file *stream = (struct ui_file *) (*args)->args[2];
-  int show = (int) (*args)->args[3];
-  type_print (VALUE_TYPE (val), varstring, stream, show);
+  value_ptr val = (value_ptr) (*args)->args[0].ptr;
+  char *varstring = (*args)->args[1].ptr;
+  struct ui_file *stream = (struct ui_file *) (*args)->args[2].ptr;
+  int show = (*args)->args[3].integer;
+  type_print (value_type (val), varstring, stream, show);
   return 1;
 }
 
@@ -173,14 +183,14 @@ GDB_val_print (struct type *type,
 {
   struct gdb_wrapper_arguments args;
 
-  args.args[0] = (char *) type;
-  args.args[1] = (char *) valaddr;
-  args.args[2] = (char *) &address;
-  args.args[3] = (char *) stream;
-  args.args[4] = (char *) format;
-  args.args[5] = (char *) deref_ref;
-  args.args[6] = (char *) recurse;
-  args.args[7] = (char *) pretty;
+  args.args[0].ptr = type;
+  args.args[1].ptr = valaddr;
+  args.args[2].ptr = &address;
+  args.args[3].ptr = stream;
+  args.args[4].integer = format;
+  args.args[5].integer = deref_ref;
+  args.args[6].integer = recurse;
+  args.args[7].integer = pretty;
 
   return call_wrapped_function ((catch_errors_ftype *) wrap_val_print, &args);
 }
@@ -190,7 +200,7 @@ wrap_val_print (char *a)
 {
   struct gdb_wrapper_arguments **args = (struct gdb_wrapper_arguments **) a;
   struct type *type;
-  char *valaddr;
+  const gdb_byte *valaddr;
   CORE_ADDR address;
   struct ui_file *stream;
   int format;
@@ -198,14 +208,14 @@ wrap_val_print (char *a)
   int recurse;
   enum val_prettyprint pretty;
 
-  type = (struct type *) (*args)->args[0];
-  valaddr = (char *) (*args)->args[1];
-  address = *(CORE_ADDR *) (*args)->args[2];
-  stream = (struct ui_file *) (*args)->args[3];
-  format = (int) (*args)->args[4];
-  deref_ref = (int) (*args)->args[5];
-  recurse = (int) (*args)->args[6];
-  pretty = (enum val_prettyprint) (*args)->args[7];
+  type = (struct type *) (*args)->args[0].ptr;
+  valaddr = (gdb_byte *) (*args)->args[1].ptr;
+  address = *(CORE_ADDR *) (*args)->args[2].ptr;
+  stream = (struct ui_file *) (*args)->args[3].ptr;
+  format = (*args)->args[4].integer;
+  deref_ref = (*args)->args[5].integer;
+  recurse = (*args)->args[6].integer;
+  pretty = (enum val_prettyprint) (*args)->args[7].integer;
 
   val_print (type, valaddr, 0, address, stream, format, deref_ref,
 	     recurse, pretty);
@@ -217,7 +227,7 @@ GDB_value_fetch_lazy (value_ptr value)
 {
   struct gdb_wrapper_arguments args;
 
-  args.args[0] = (char *) value;
+  args.args[0].ptr = value;
   return call_wrapped_function ((catch_errors_ftype *) wrap_value_fetch_lazy, &args);
 }
 
@@ -226,7 +236,7 @@ wrap_value_fetch_lazy (char *a)
 {
   struct gdb_wrapper_arguments **args = (struct gdb_wrapper_arguments **) a;
 
-  value_fetch_lazy ((value_ptr) (*args)->args[0]);
+  value_fetch_lazy ((value_ptr) (*args)->args[0].ptr);
   return 1;
 }
 
@@ -235,13 +245,13 @@ GDB_evaluate_expression (struct expression *exp, value_ptr *value)
 {
   struct gdb_wrapper_arguments args;
   gdb_result result;
-  args.args[0] = (char *) exp;
+  args.args[0].ptr = exp;
 
   result = call_wrapped_function ((catch_errors_ftype *) wrap_evaluate_expression, &args);
   if (result != GDB_OK)
     return result;
 
-  *value = (value_ptr) args.result;
+  *value = (value_ptr) args.result.ptr;
   return GDB_OK;
 }
 
@@ -250,8 +260,8 @@ wrap_evaluate_expression (char *a)
 {
   struct gdb_wrapper_arguments **args = (struct gdb_wrapper_arguments **) a;
 
-  (*args)->result =
-    (char *) evaluate_expression ((struct expression *) (*args)->args[0]);
+  (*args)->result.ptr =
+    evaluate_expression ((struct expression *) (*args)->args[0].ptr);
   return 1;
 }
 
@@ -264,14 +274,14 @@ GDB_value_equal (val1, val2, result)
   struct gdb_wrapper_arguments args;
   gdb_result r;
 
-  args.args[0] = (char *) val1;
-  args.args[1] = (char *) val2;
+  args.args[0].ptr = val1;
+  args.args[1].ptr = val2;
 
   r = call_wrapped_function ((catch_errors_ftype *) wrap_value_equal, &args);
   if (r != GDB_OK)
     return r;
 
-  *result = (int) args.result;
+  *result = args.result.integer;
   return GDB_OK;
 }
 
@@ -281,10 +291,10 @@ wrap_value_equal (char *a)
   struct gdb_wrapper_arguments **args = (struct gdb_wrapper_arguments **) a;
   value_ptr val1, val2;
 
-  val1 = (value_ptr) (*args)->args[0];
-  val2 = (value_ptr) (*args)->args[1];
+  val1 = (value_ptr) (*args)->args[0].ptr;
+  val2 = (value_ptr) (*args)->args[1].ptr;
 
-  (*args)->result = (char *) value_equal (val1, val2);
+  (*args)->result.integer = value_equal (val1, val2);
   return 1;
 }
 
@@ -295,15 +305,15 @@ GDB_parse_exp_1 (char **stringptr, struct block *block,
   struct gdb_wrapper_arguments args;
   gdb_result r;
 
-  args.args[0] = (char *) stringptr;
-  args.args[1] = (char *) block;
-  args.args[2] = (char *) comma;
+  args.args[0].ptr = stringptr;
+  args.args[1].ptr = block;
+  args.args[2].integer = comma;
 
   r = call_wrapped_function ((catch_errors_ftype *) wrap_parse_exp_1, &args);
   if (r != GDB_OK)
     return r;
 
-  *result = (struct expression *) args.result;
+  *result = (struct expression *) args.result.ptr;
   return GDB_OK;
 }
 
@@ -315,11 +325,11 @@ wrap_parse_exp_1 (char *opaque_arg)
   char **stringptr;
   int comma;
 
-  stringptr = (char **) (*args)->args[0];
-  block = (struct block *) (*args)->args[1];
-  comma = (int) (*args)->args[2];
+  stringptr = (char **) (*args)->args[0].ptr;
+  block = (struct block *) (*args)->args[1].ptr;
+  comma = (*args)->args[2].integer;
 
-  (*args)->result = (char *) parse_exp_1 (stringptr, block, comma);
+  (*args)->result.ptr = parse_exp_1 (stringptr, block, comma);
   return 1;
 }
 
@@ -329,13 +339,13 @@ GDB_evaluate_type (struct expression *exp, value_ptr *result)
   struct gdb_wrapper_arguments args;
   gdb_result r;
 
-  args.args[0] = (char *) exp;
+  args.args[0].ptr = exp;
 
   r = call_wrapped_function ((catch_errors_ftype *) wrap_evaluate_type, &args);
   if (r != GDB_OK)
     return r;
 
-  *result = (value_ptr) args.result;
+  *result = (value_ptr) args.result.ptr;
   return GDB_OK;
 }
 
@@ -345,8 +355,8 @@ wrap_evaluate_type (char *opaque_arg)
   struct gdb_wrapper_arguments **args = (struct gdb_wrapper_arguments **) opaque_arg;
   struct expression *exp;
 
-  exp = (struct expression *) (*args)->args[0];
-  (*args)->result = (char *) evaluate_type (exp);
+  exp = (struct expression *) (*args)->args[0].ptr;
+  (*args)->result.ptr = evaluate_type (exp);
   return 1;
 }
 
@@ -356,13 +366,13 @@ GDB_block_for_pc (CORE_ADDR pc, struct block **result)
   struct gdb_wrapper_arguments args;
   gdb_result r;
 
-  args.args[0] = (char *) &pc;
+  args.args[0].ptr = &pc;
 
   r = call_wrapped_function ((catch_errors_ftype *) wrap_block_for_pc, &args);
   if (r != GDB_OK)
     return r;
 
-  *result = (struct block *) args.result;
+  *result = (struct block *) args.result.ptr;
   return GDB_OK;
 }
 
@@ -372,8 +382,8 @@ wrap_block_for_pc (char *opaque_arg)
   struct gdb_wrapper_arguments **args = (struct gdb_wrapper_arguments **) opaque_arg;
   CORE_ADDR pc;
 
-  pc = *(CORE_ADDR *) (*args)->args[0];
-  (*args)->result = (char *) block_for_pc (pc);
+  pc = *(CORE_ADDR *) (*args)->args[0].ptr;
+  (*args)->result.ptr = block_for_pc (pc);
   return 1;
 }
 
@@ -383,13 +393,13 @@ GDB_block_innermost_frame (struct block *block, struct frame_info **result)
   struct gdb_wrapper_arguments args;
   gdb_result r;
 
-  args.args[0] = (char *) block;
+  args.args[0].ptr = block;
 
   r = call_wrapped_function ((catch_errors_ftype *) wrap_block_innermost_frame, &args);
   if (r != GDB_OK)
     return r;
 
-  *result = (struct frame_info *) args.result;
+  *result = (struct frame_info *) args.result.ptr;
   return GDB_OK;
 }
 
@@ -399,8 +409,8 @@ wrap_block_innermost_frame (char *opaque_arg)
   struct gdb_wrapper_arguments **args = (struct gdb_wrapper_arguments **) opaque_arg;
   struct block *block;
 
-  block = (struct block *) (*args)->args[0];
-  (*args)->result = (char *) block_innermost_frame (block);
+  block = (struct block *) (*args)->args[0].ptr;
+  (*args)->result.ptr = block_innermost_frame (block);
   return 1;
 }
 
@@ -429,13 +439,13 @@ GDB_value_ind (value_ptr val, value_ptr *rval)
   struct gdb_wrapper_arguments args;
   gdb_result r;
 
-  args.args[0] = (char *) val;
+  args.args[0].ptr = val;
 
   r = call_wrapped_function ((catch_errors_ftype *) wrap_value_ind, &args);
   if (r != GDB_OK)
     return r;
 
-  *rval = (value_ptr) args.result;
+  *rval = (value_ptr) args.result.ptr;
   return GDB_OK;
 }
 
@@ -445,8 +455,8 @@ wrap_value_ind (char *opaque_arg)
   struct gdb_wrapper_arguments **args = (struct gdb_wrapper_arguments **) opaque_arg;
   value_ptr val;
 
-  val = (value_ptr) (*args)->args[0];
-  (*args)->result = (char *) value_ind (val);
+  val = (value_ptr) (*args)->args[0].ptr;
+  (*args)->result.ptr = value_ind (val);
   return 1;
 }
 
@@ -456,15 +466,15 @@ GDB_value_slice (value_ptr val, int low, int num, value_ptr *rval)
   struct gdb_wrapper_arguments args;
   gdb_result r;
 
-  args.args[0] = (char *) val;
-  args.args[1] = (char *) &low;
-  args.args[2] = (char *) &num;
+  args.args[0].ptr = val;
+  args.args[1].integer = low;
+  args.args[2].integer = num;
 
   r = call_wrapped_function ((catch_errors_ftype *) wrap_value_slice, &args);
   if (r != GDB_OK)
     return r;
 
-  *rval = (value_ptr) args.result;
+  *rval = (value_ptr) args.result.ptr;
   return GDB_OK;
 }
 
@@ -475,10 +485,10 @@ wrap_value_slice (char *opaque_arg)
   value_ptr val;
   int low, num;
 
-  val = (value_ptr) (*args)->args[0];
-  low = *(int *) (*args)->args[1];
-  num = *(int *) (*args)->args[2];
-  (*args)->result = (char *) value_slice (val, low, num);
+  val = (value_ptr) (*args)->args[0].ptr;
+  low = (*args)->args[1].integer;
+  num = (*args)->args[2].integer;
+  (*args)->result.ptr = value_slice (val, low, num);
   return 1;
 }
 
@@ -490,14 +500,14 @@ GDB_value_coerce_array (val, rval)
   struct gdb_wrapper_arguments args;
   gdb_result r;
 
-  args.args[0] = (char *) val;
+  args.args[0].ptr = val;
 
   r = call_wrapped_function ((catch_errors_ftype *) wrap_value_coerce_array,
 			     &args);
   if (r != GDB_OK)
     return r;
 
-  *rval = (value_ptr) args.result;
+  *rval = (value_ptr) args.result.ptr;
   return GDB_OK;
 }
 
@@ -507,8 +517,8 @@ wrap_value_coerce_array (char *opaque_arg)
   struct gdb_wrapper_arguments **args = (struct gdb_wrapper_arguments **) opaque_arg;
   value_ptr val;
 
-  val = (value_ptr) (*args)->args[0];
-  (*args)->result = (char *) value_coerce_array (val);
+  val = (value_ptr) (*args)->args[0].ptr;
+  (*args)->result.ptr = value_coerce_array (val);
   return 1;
 }
 
@@ -523,16 +533,16 @@ GDB_value_struct_elt (value_ptr *argp,
   struct gdb_wrapper_arguments argss;
   gdb_result r;
 
-  argss.args[0] = (char *) argp;
-  argss.args[1] = (char *) args;
-  argss.args[2] = name;
-  argss.args[3] = (char *) static_memfunc;
-  argss.args[4] = err;
+  argss.args[0].ptr = argp;
+  argss.args[1].ptr = args;
+  argss.args[2].ptr = name;
+  argss.args[3].ptr = static_memfunc;
+  argss.args[4].ptr = err;
   r = call_wrapped_function ((catch_errors_ftype *) wrap_value_struct_elt, &argss);
   if (r != GDB_OK)
     return r;
 
-  *rval = (value_ptr) argss.result;
+  *rval = (value_ptr) argss.result.ptr;
   return GDB_OK;
 }
 
@@ -545,13 +555,13 @@ wrap_value_struct_elt (char *opaque_arg)
   int *static_memfunc;
   char *err;
 
-  argp = (value_ptr *) (*argss)->args[0];
-  args = (value_ptr *) (*argss)->args[1];
-  name = (*argss)->args[2];
-  static_memfunc = (int *) (*argss)->args[3];
-  err = (*argss)->args[4];
+  argp = (value_ptr *) (*argss)->args[0].ptr;
+  args = (value_ptr *) (*argss)->args[1].ptr;
+  name = (char *) (*argss)->args[2].ptr;
+  static_memfunc = (int *) (*argss)->args[3].ptr;
+  err = (char *) (*argss)->args[4].ptr;
 
-  (*argss)->result = (char *) value_struct_elt (argp, args, name, static_memfunc, err);
+  (*argss)->result.ptr = value_struct_elt (argp, args, name, static_memfunc, err);
   return 1;
 }
 
@@ -561,14 +571,14 @@ GDB_value_cast (struct type *type, value_ptr val, value_ptr *rval)
   struct gdb_wrapper_arguments args;
   gdb_result r;
 
-  args.args[0] = (char *) type;
-  args.args[1] = (char *) val;
+  args.args[0].ptr = type;
+  args.args[1].ptr = val;
 
   r = call_wrapped_function ((catch_errors_ftype *) wrap_value_cast, &args);
   if (r != GDB_OK)
     return r;
 
-  *rval = (value_ptr) args.result;
+  *rval = (value_ptr) args.result.ptr;
   return GDB_OK;
 }
 
@@ -579,9 +589,9 @@ wrap_value_cast (char *opaque_arg)
   value_ptr val;
   struct type *type;
 
-  type = (struct type *) (*args)->args[0];
-  val = (value_ptr) (*args)->args[1];
-  (*args)->result = (char *) value_cast (type, val);
+  type = (struct type *) (*args)->args[0].ptr;
+  val = (value_ptr) (*args)->args[1].ptr;
+  (*args)->result.ptr = value_cast (type, val);
 
   return 1;
 }
@@ -592,13 +602,13 @@ GDB_get_frame_block (struct frame_info *fi, struct block **rval)
   struct gdb_wrapper_arguments args;
   gdb_result r;
 
-  args.args[0] = (char *) fi;
+  args.args[0].ptr = fi;
 
   r = call_wrapped_function ((catch_errors_ftype *) wrap_get_frame_block, &args);
   if (r != GDB_OK)
     return r;
 
-  *rval = (struct block *) args.result;
+  *rval = (struct block *) args.result.ptr;
   return GDB_OK;
 }
 
@@ -608,8 +618,8 @@ wrap_get_frame_block (char *opaque_arg)
   struct gdb_wrapper_arguments **args = (struct gdb_wrapper_arguments **) opaque_arg;
   struct frame_info *fi;
 
-  fi = (struct frame_info *) (*args)->args[0];
-  (*args)->result = (char *) get_frame_block (fi, NULL);
+  fi = (struct frame_info *) (*args)->args[0].ptr;
+  (*args)->result.ptr = get_frame_block (fi, NULL);
 
   return 1;
 }
@@ -620,12 +630,12 @@ GDB_get_prev_frame (struct frame_info *fi, struct frame_info **result)
   struct gdb_wrapper_arguments args;
   gdb_result r;
 
-  args.args[0] = (char *) fi;
+  args.args[0].ptr = fi;
   r = call_wrapped_function ((catch_errors_ftype *) wrap_get_prev_frame, &args);
   if (r != GDB_OK)
     return r;
 
-  *result = (struct frame_info *) args.result;
+  *result = (struct frame_info *) args.result.ptr;
   return GDB_OK;
 }
 
@@ -633,9 +643,9 @@ static int
 wrap_get_prev_frame (char *opaque_arg)
 {
   struct gdb_wrapper_arguments **args = (struct gdb_wrapper_arguments **) opaque_arg;
-  struct frame_info *fi = (struct frame_info *) (*args)->args[0];
+  struct frame_info *fi = (struct frame_info *) (*args)->args[0].ptr;
 
-  (*args)->result = (char *) get_prev_frame (fi);
+  (*args)->result.ptr = get_prev_frame (fi);
   return 1;
 }
 
@@ -645,12 +655,12 @@ GDB_get_next_frame (struct frame_info *fi, struct frame_info **result)
   struct gdb_wrapper_arguments args;
   gdb_result r;
 
-  args.args[0] = (char *) fi;
+  args.args[0].ptr = fi;
   r = call_wrapped_function ((catch_errors_ftype *) wrap_get_next_frame, &args);
   if (r != GDB_OK)
     return r;
 
-  *result = (struct frame_info *) args.result;
+  *result = (struct frame_info *) args.result.ptr;
   return GDB_OK;
 }
 
@@ -658,9 +668,9 @@ static int
 wrap_get_next_frame (char *opaque_arg)
 {
   struct gdb_wrapper_arguments **args = (struct gdb_wrapper_arguments **) opaque_arg;
-  struct frame_info *fi = (struct frame_info *) (*args)->args[0];
+  struct frame_info *fi = (struct frame_info *) (*args)->args[0].ptr;
 
-  (*args)->result = (char *) get_next_frame (fi);
+  (*args)->result.ptr = get_next_frame (fi);
   return 1;
 }
 
@@ -671,15 +681,15 @@ GDB_find_relative_frame (struct frame_info *fi, int *start,
   struct gdb_wrapper_arguments args;
   gdb_result r;
 
-  args.args[0] = (char *) fi;
-  args.args[1] = (char *) start;
+  args.args[0].ptr = fi;
+  args.args[1].ptr = start;
 
   r = call_wrapped_function ((catch_errors_ftype *) wrap_find_relative_frame, 
 			     &args);
   if (r != GDB_OK)
     return r;
 
-  *result = (struct frame_info *) args.result;
+  *result = (struct frame_info *) args.result.ptr;
   return GDB_OK;
 }
 
@@ -687,10 +697,10 @@ static int
 wrap_find_relative_frame (char *opaque_arg)
 {
   struct gdb_wrapper_arguments **args = (struct gdb_wrapper_arguments **) opaque_arg;
-  struct frame_info *fi = (struct frame_info *) (*args)->args[0];
-  int *start = (int *) (*args)->args[1];
+  struct frame_info *fi = (struct frame_info *) (*args)->args[0].ptr;
+  int *start = (int *) (*args)->args[1].ptr;
 
-  (*args)->result = (char *) find_relative_frame (fi, start);
+  (*args)->result.ptr = find_relative_frame (fi, start);
   return 1;
 }
 
@@ -705,7 +715,7 @@ GDB_get_current_frame (struct frame_info **result)
   if (r != GDB_OK)
     return r;
 
-  *result = (struct frame_info *) args.result;
+  *result = (struct frame_info *) args.result.ptr;
   return GDB_OK;
 }
 
@@ -714,7 +724,37 @@ wrap_get_current_frame (char *opaque_arg)
 {
   struct gdb_wrapper_arguments **args = (struct gdb_wrapper_arguments **) opaque_arg;
 
-  (*args)->result = (char *) get_current_frame ();
+  (*args)->result.ptr = get_current_frame ();
   return 1;
 }
 
+gdb_result
+GDB_varobj_update (struct varobj **varp, struct varobj ***changelist,
+		   int explicit, int *result)
+{
+  struct gdb_wrapper_arguments args;
+  gdb_result r;
+
+  args.args[0].ptr = varp;
+  args.args[1].ptr = changelist;
+  args.args[2].integer = explicit;
+
+  r = call_wrapped_function ((catch_errors_ftype *) wrap_varobj_update, &args);
+  if (r != GDB_OK)
+    return r;
+
+  *result = args.result.integer;
+  return GDB_OK;
+}
+
+static int wrap_varobj_update (char *opaque_arg)
+{
+  struct gdb_wrapper_arguments **args
+    = (struct gdb_wrapper_arguments **) opaque_arg;
+  struct varobj **varp = (struct varobj **) (*args)->args[0].ptr;
+  struct varobj ***changelist = (struct varobj ***) (*args)->args[1].ptr;
+  int explicit = (*args)->args[2].integer;
+
+  (*args)->result.integer = varobj_update (varp, changelist, explicit);
+  return 1;
+}
